//
// file: primitive3D_sphere_grid.fx
// version: v1.5
// author: Ren712
//

//--------------------------------------------------------------------------------------
// Settings
//--------------------------------------------------------------------------------------
float3 sElementPosition = float3(0, 0, 0);
float3 sElementRotation = float3(0, 0, 0);
float4 sElementColor = float4(0, 0, 0, 0);
float3 sElementSize = float3(1,1,1);
float2 sSubdivUnit = float2(0.08,0.08);

bool sFlipTexture = false;
bool sFlipVertex = false;
bool sIsBillboard = false;
bool sFogEnable = false;
int fCullMode = 1;

float2 gDistFade = float2(250, 150);
float fDestBlend = 2;

//--------------------------------------------------------------------------------------
// Variables set by MTA
//--------------------------------------------------------------------------------------
float4x4 gWorldViewProjection : WORLDVIEWPROJECTION;
float4x4 gProjection : PROJECTION;
float4x4 gView : VIEW;
float4x4 gViewInverse : VIEWINVERSE;
float3 gCameraPosition : CAMERAPOSITION;
float4 gFogColor < string renderState="FOGCOLOR"; >;
static const float PI = 3.14159265f;
float gTime : TIME;
int gCapsMaxAnisotropy < string deviceCaps="MaxAnisotropy"; >;
int CUSTOMFLAGS < string skipUnusedParameters = "yes"; >;

//--------------------------------------------------------------------------------------
// Structures
//--------------------------------------------------------------------------------------
struct VSInput
{
    float3 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float4 Diffuse : COLOR0;
};

struct PSInput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float3 WorldPos : TEXCOORD1;
    float3 Normal : TEXCOORD2;
    float DistFade : TEXCOORD3;
    float4 Diffuse : COLOR0;
};

//--------------------------------------------------------------------------------------
// Create world matrix with world position and euler rotation
//--------------------------------------------------------------------------------------
float4x4 createWorldMatrix(float3 pos, float3 rot)
{
    float4x4 eleMatrix = {
        float4(cos(rot.z) * cos(rot.y) - sin(rot.z) * sin(rot.x) * sin(rot.y), 
                cos(rot.y) * sin(rot.z) + cos(rot.z) * sin(rot.x) * sin(rot.y), -cos(rot.x) * sin(rot.y), 0),
        float4(-cos(rot.x) * sin(rot.z), cos(rot.z) * cos(rot.x), sin(rot.x), 0),
        float4(cos(rot.z) * sin(rot.y) + cos(rot.y) * sin(rot.z) * sin(rot.x), sin(rot.z) * sin(rot.y) - 
                cos(rot.z) * cos(rot.y) * sin(rot.x), cos(rot.x) * cos(rot.y), 0),
        float4(pos.x,pos.y,pos.z, 1),
    };
    return eleMatrix;
}

//--------------------------------------------------------------------------------------
// Transform quad into sphere 
//--------------------------------------------------------------------------------------
float3 getSpherePosition(float3 inPosition, float3 scale)
{
    float3 outPosition;
    outPosition.z = cos(2 * inPosition.x * PI) / 2;
	outPosition.x = sin(2 * inPosition.x * PI) / 2;
	outPosition.xz *= cos(inPosition.y * PI);
	outPosition.y = sin(inPosition.y * PI) / 2;
    return outPosition * scale;
}

//--------------------------------------------------------------------------------------
// Vertex Shader 
//--------------------------------------------------------------------------------------
PSInput VertexShaderFunction(VSInput VS)
{
    PSInput PS = (PSInput)0;

    // set proper position and scale of the quad
    VS.Position.xyz = float3(- 0.5 + VS.TexCoord.xy, 0);
	
    // shape the sphere
    float3 scaleNorm = normalize(sElementSize);
    float3 resultPos = getSpherePosition(VS.Position.xyz, scaleNorm);
	
    // Calculate normals
    float3 Normal = normalize(resultPos);
	
    // scale the sphere
    VS.Position.xyz = resultPos * length(sElementSize);	
	
    if (!sFlipTexture) VS.TexCoord.y = 1 - VS.TexCoord.y;
    if (sFlipVertex) VS.Position.xyz = VS.Position.xzy;
	
    // create WorldMatrix for the quad
    float4x4 sWorld = createWorldMatrix(sElementPosition, sElementRotation);

    // calculate screen position of the vertex
    float4 wPos = mul(float4( VS.Position, 1), sWorld);
    PS.WorldPos = wPos.xyz;
    float4 vPos = mul(wPos, gView);
    PS.Position = mul(vPos, gProjection);
	
    // get world normal, binormal and tangent
    PS.Normal = mul(Normal, (float3x3)sWorld).xyz;
	
    // get clip values
    float nearClip = - gProjection[3][2] / gProjection[2][2];
    float farClip = (gProjection[3][2] / (1 - gProjection[2][2]));
	
    // fade
    float DistFromCam = distance(gCameraPosition, sElementPosition.xyz);
    float elementSize = max(max(sElementSize.x, sElementSize.y), sElementSize.z);
    float2 DistFade = float2(min(gDistFade.x, farClip - elementSize / 2), min(gDistFade.y, farClip - elementSize /2 - (gDistFade.x - gDistFade.y)));
    PS.DistFade = saturate((DistFromCam - DistFade.x)/(DistFade.y - DistFade.x));

    // pass texCoords and vertex color to PS
    float2 uvMul = (1 / sSubdivUnit);
    PS.TexCoord = (VS.TexCoord * uvMul);
    PS.Diffuse = VS.Diffuse * sElementColor;
    return PS;
}

float2 getReflectionCoords2(float3 dir, float2 div)
{
    return float2(((atan2(dir.x, dir.z) / (PI * div.x)) + 1) / 2,  (acos(- dir.y) / (PI * div.y)));
}

//--------------------------------------------------------------------------------------
// Pixel shaders 
//--------------------------------------------------------------------------------------
float4 PixelShaderFunctionSM3(PSInput PS) : COLOR0
{
    // create rotation matrix (for the cube texture)
    float3x3 sRotation = (float3x3)createWorldMatrix(sElementPosition, sElementRotation);
	
    // rotate the lightVector
    float3 Normal = mul(sRotation, normalize(PS.Normal));
	
    // create grid
    float2 texCoord = getReflectionCoords2(-Normal.xzy,float2(1,1));
	
    float2 fillMode;
    fillMode.x = sin((max(0, fmod((texCoord.x / sSubdivUnit.x), 1) - 0.9) / 0.1) * PI);
    fillMode.y = sin((max(0, fmod((texCoord.y / sSubdivUnit.y), 1) - 0.9) / 0.1) * PI);
	
    float4 finalColor = 1;
    finalColor.a = (fillMode.x + fillMode.y);
	
    // multiply by vertex color
    finalColor *= PS.Diffuse;

    // apply distFade
    finalColor.a *= saturate( PS.DistFade );

    return saturate(finalColor);
}

float4 PixelShaderFunction(PSInput PS) : COLOR0
{
    // create grid	
    float2 fillMode;
    fillMode.x = sin((max(0, fmod((PS.TexCoord.x), 1) - 0.9) / 0.1) * PI);
    fillMode.y = sin((max(0, fmod((PS.TexCoord.y), 1) - 0.9) / 0.1) * PI);
	
    float4 finalColor = 1;
    finalColor.a = (fillMode.x + fillMode.y);
	
    // multiply by vertex color
    finalColor *= PS.Diffuse;

    // apply distFade
    finalColor.a *= saturate( PS.DistFade );

    return saturate(finalColor);
}

//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique dxDrawPrimitive3Dfx_sphere_grid
{
  pass P0
  {
    ZEnable = true;
    ZFunc = LessEqual;
    ZWriteEnable = false;
    CullMode = fCullMode;
    ShadeMode = Gouraud;
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = fDestBlend;
    AlphaTestEnable = true;
    AlphaRef = 1;
    AlphaFunc = GreaterEqual;
    Lighting = false;
    FogEnable = false;
    VertexShader = compile vs_3_0 VertexShaderFunction();
    PixelShader  = compile ps_3_0 PixelShaderFunctionSM3();
  }
}

technique dxDrawPrimitive3Dfx_sphere_grid_SM2
{
  pass P0
  {
    ZEnable = true;
    ZFunc = LessEqual;
    ZWriteEnable = false;
    CullMode = fCullMode;
    ShadeMode = Gouraud;
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = fDestBlend;
    AlphaTestEnable = true;
    AlphaRef = 1;
    AlphaFunc = GreaterEqual;
    Lighting = false;
    FogEnable = false;
    VertexShader = compile vs_2_0 VertexShaderFunction();
    PixelShader  = compile ps_2_0 PixelShaderFunction();
  }
}

// Fallback
technique fallback
{
  pass P0
  {
    // Just draw normally
  }
}
